@@ -5,11 +5,12 @@
 #'   prediction interval for the expected time to reach the target
 #'   number of events.
 #'
-#' @param df The subject-level enrollment and event data,
-#'   including \code{trialsdt}, \code{randdt}, \code{cutoffdt},
+#' @param df The subject-level enrollment and event data, including
+#'   \code{trialsdt}, \code{usubjid}, \code{randdt}, \code{cutoffdt},
 #'   \code{time}, \code{event}, and \code{dropout}. The data should also
-#'   include \code{treatment} for by-treatment prediction. By default, it
-#'   is set to \code{NULL} for event prediction at the design stage.
+#'   include \code{treatment} coded as 1, 2, and so on, and
+#'   \code{treatment_description} for by-treatment prediction. By default,
+#'   it is set to \code{NULL} for event prediction at the design stage.
 #' @param target_d The target number of events to reach in the study.
 #' @param newSubjects The enrollment data for new subjects including
 #'   \code{draw} and \code{arrivalTime}. The data should also include
@@ -58,31 +59,19 @@
 #' To ensure successful event prediction at the design stage, it is
 #' important to provide the \code{newSubjects} data set.
 #'
-#' To specify the event model used during the design-stage event
-#' prediction, the \code{event_fit} be a list with one element
-#' per treatment. For each treatment, the element should include \code{w}
-#' to specify the weight of the treatment in a randomization block,
-#' \code{model} to specify the event model
-#' (exponential, weibull, log-normal, or piecewise exponential),
-#' \code{theta} and \code{vtheta} to indicate
-#' the parameter values and the covariance matrix.
-#' For the piecewise exponential event model, the list
-#' should also include \code{piecewiseSurvivalTime} to indicate
+#' To specify the event (dropout) model used during the design-stage event
+#' prediction, the \code{event_fit} (\code{dropout_fit}) be a list with
+#' one element per treatment. For each treatment, the element should
+#' include \code{w} to specify the weight of the treatment in a
+#' randomization block, \code{model} to specify the event model
+#' (exponential, weibull, log-logistic, log-normal, or piecewise
+#' exponential), \code{theta} and \code{vtheta} to indicate
+#' the parameter values and the covariance matrix. For the piecewise
+#' exponential event (dropout) model, the list should also include
+#' \code{piecewiseSurvivalTime} \code{piecewiseDropoutTime} to indicate
 #' the location of knots. It should be noted that the model averaging
 #' and spline options are not appropriate for use during the design stage.
 #'
-#' To specify the dropout model used during the design stage
-#' event prediction, the \code{dropout_fit} should be a list
-#' with one element per treatment. For each treatment, the element
-#' should include \code{w} to specify the weight of the treatment
-#' in a randomization block, \code{model} to specify the dropout model
-#' (exponential, weibull, log-normal, or piecewise exponential),
-#' \code{theta} and \code{vtheta} to indicate
-#' the parameter values and the covariance matrix.
-#' For the piecewise exponential dropout model, the list
-#' should also include \code{piecewiseDropoutTime} to indicate
-#' the location of knots.
-#'
 #' Following the commencement of the trial, we obtain the event
 #' model fit and the dropout model fit based on the observed data,
 #' denoted as \code{event_fit} and \code{dropout_fit}, respectively.
@@ -127,7 +116,7 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
   erify::check_n(target_d)
   if (!is.null(newSubjects)) erify::check_class(newSubjects, "data.frame")
   if (is.null(df) && is.null(newSubjects)) {
-    stop("At least one of df and newSubjects must be specified.")
+    stop("At least one of df and newSubjects must be specified")
   }
 
   erify::check_bool(by_treatment)
@@ -145,12 +134,39 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
 
     if (!is.null(df) && !is.null(newSubjects) &&
         length(table(df$treatment)) != length(table(newSubjects$treatment))) {
-      stop("Number of treatments must match between df and newSubjects.")
+      stop("Number of treatments must match between df and newSubjects")
     }
+
+    if (!is.null(df)) {
+      if (!("treatment_description" %in% names(df))) {
+        df <- df %>% dplyr::mutate(
+            treatment_description = paste0("Treatment ", .data$treatment))
+      }
+
+      df$treatment_description = stats::reorder(
+        as.factor(df$treatment_description), df$treatment)
+    }
+
+    if (!is.null(newSubjects)) {
+      if (!("treatment_description" %in% names(newSubjects))) {
+      newSubjects <- newSubjects %>% dplyr::mutate(
+          treatment_description = paste0("Treatment ", .data$treatment))
+      }
+
+      newSubjects$treatment_description = stats::reorder(
+        as.factor(newSubjects$treatment_description), newSubjects$treatment)
+    }
+
   } else {  # treat as a special case of by-treatment calculation
     ngroups = 1
-    if (!is.null(df)) df$treatment = 1
-    if (!is.null(newSubjects)) newSubjects$treatment = 1
+    if (!is.null(df)) {
+      df$treatment = 1
+      df$treatment_description = "Overall"
+    }
+    if (!is.null(newSubjects)) {
+      newSubjects$treatment = 1
+      newSubjects$treatment_description = "Overall"
+    }
   }
 
   if (ngroups == 1) {
@@ -167,22 +183,22 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
   }
 
   if (length(event_fit) != ngroups) {
-    stop("event_fit must be a list with one element per treatment.")
+    stop("event_fit must be a list with one element per treatment")
   }
 
   # check event_fit model
   if (!is.null(df)) {
     for (j in 1:ngroups) {
       erify::check_content(tolower(event_fit[[j]]$model),
-                           c("exponential", "weibull", "log-normal",
-                             "piecewise exponential", "model averaging",
-                             "spline"))
+                           c("exponential", "weibull", "log-logistic",
+                             "log-normal", "piecewise exponential",
+                             "model averaging", "spline"))
     }
   } else {
     for (j in 1:ngroups) {
       erify::check_content(tolower(event_fit[[j]]$model),
-                           c("exponential", "weibull", "log-normal",
-                             "piecewise exponential"))
+                           c("exponential", "weibull", "log-logistic",
+                             "log-normal", "piecewise exponential"))
     }
   }
 
@@ -201,9 +217,12 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
 
     if ((model == "exponential" && p != 1) ||
         (model == "weibull" && p != 2) ||
+        (model == "log-logistic" && p != 2) ||
         (model == "log-normal" && p != 2) ||
         (model == "piecewise exponential" &&
-         p != length(event_fit[[j]]$piecewiseSurvivalTime))) {
+         p != length(event_fit[[j]]$piecewiseSurvivalTime)) ||
+        (model == "model averaging" && p != 4) ||
+        (model == "spline" && p != length(event_fit[[j]]$knots))) {
       stop(paste("Length of theta must be compatible with model",
                  "in event_fit"))
     }
@@ -232,14 +251,23 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
     }
 
     if (length(dropout_fit) != ngroups) {
-      stop("dropout_fit must be a list with one element per treatment.")
+      stop("dropout_fit must be a list with one element per treatment")
     }
 
     # check dropout_fit model
-    for (j in 1:ngroups) {
-      erify::check_content(tolower(dropout_fit[[j]]$model),
-                           c("exponential", "weibull", "log-normal",
-                             "piecewise exponential"))
+    if (!is.null(df)) {
+      for (j in 1:ngroups) {
+        erify::check_content(tolower(dropout_fit[[j]]$model),
+                             c("exponential", "weibull", "log-logistic",
+                               "log-normal", "piecewise exponential",
+                               "model averaging", "spline"))
+      }
+    } else {
+      for (j in 1:ngroups) {
+        erify::check_content(tolower(dropout_fit[[j]]$model),
+                             c("exponential", "weibull", "log-logistic",
+                               "log-normal", "piecewise exponential"))
+      }
     }
 
     # check dropout_fit parameters
@@ -257,9 +285,12 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
 
       if ((model == "exponential" && p != 1) ||
           (model == "weibull" && p != 2) ||
+          (model == "log-logistic" && p != 2) ||
           (model == "log-normal" && p != 2) ||
           (model == "piecewise exponential" &&
-           p != length(dropout_fit[[j]]$piecewiseDropoutTime))) {
+           p != length(dropout_fit[[j]]$piecewiseDropoutTime)) ||
+          (model == "model averaging" && p != 4) ||
+          (model == "spline" && p != length(dropout_fit[[j]]$knots))) {
         stop(paste("Length of theta must be compatible with model",
                    "in dropout_fit"))
       }
@@ -292,7 +323,7 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
   erify::check_bool(showplot)
 
   if (!(showEnrollment | showEvent | showDropout | showOngoing)) {
-    stop("At least one parameter must be given for prediction plot.")
+    stop("At least one parameter must be given for prediction plot")
   }
 
 
@@ -309,35 +340,41 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
     t0 = as.numeric(cutoffdt - trialsdt + 1)
 
     if (any(df$randdt < trialsdt)) {
-      stop("randdt must be greater than or equal to trialsdt.")
+      stop("randdt must be greater than or equal to trialsdt")
     }
 
     if (any(df$randdt > cutoffdt)) {
-      stop("randdt must be less than or equal to cutoffdt.")
+      stop("randdt must be less than or equal to cutoffdt")
     }
 
     if (any(df$time < 1)) {
-      stop("time must be greater than or equal to 1.")
+      stop("time must be greater than or equal to 1")
     }
 
     if (any(df$event == 1 & df$dropout == 1)) {
-      stop("event and dropout cannot both be equal to 1 simultaneously.")
+      stop("event and dropout cannot both be equal to 1 simultaneously")
     }
 
     if (any(df$time > as.numeric(cutoffdt - df$randdt + 1))) {
-      stop("time must be less than or equal to cutoffdt - randdt + 1.")
+      stop("time must be less than or equal to cutoffdt - randdt + 1")
     }
 
     df <- df %>%
       dplyr::mutate(arrivalTime = as.numeric(.data$randdt - trialsdt + 1),
                     totalTime = .data$arrivalTime + .data$time - 1)
 
+    if (!("usubjid" %in% names(df))) {
+      df$usubjid = paste0("A-", 100000 + (1:nrow(df)))
+    }
+
     # subset to extract ongoing subjects
     ongoingSubjects <- df %>%
       dplyr::filter(.data$event == 0 & .data$dropout == 0)
 
+    usubjidOngoing <- ongoingSubjects$usubjid
     arrivalTimeOngoing <- ongoingSubjects$arrivalTime
     treatmentOngoing <- ongoingSubjects$treatment
+    treatment_descriptionOngoing <- ongoingSubjects$treatment_description
     time0Ongoing <- ongoingSubjects$time
     tp = min(ongoingSubjects$totalTime) # tp <= t0
     cutofftpdt <- as.Date(tp - 1, origin = trialsdt)
@@ -452,32 +489,42 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
     if (!is.null(df)) {
       # add overall treatment
       df2 <- df %>%
-        dplyr::bind_rows(df %>% dplyr::mutate(treatment = 9999))
+        dplyr::bind_rows(df %>% dplyr::mutate(
+          treatment = 9999, treatment_description = "Overall"))
 
       sum_by_trt <- df2 %>%
-        dplyr::group_by(.data$treatment) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description) %>%
         dplyr::summarise(n0 = dplyr::n(),
                          d0 = sum(.data$event),
                          c0 = sum(.data$dropout),
-                         r0 = sum(!(.data$event | .data$dropout)))
-    } else {
-      sum_by_trt <- dplyr::tibble(treatment = c(1:ngroups, 9999),
-                                  n0 = 0, d0 = 0, c0 = 0, r0 = 0)
+                         r0 = sum(!(.data$event | .data$dropout)),
+                         .groups = "drop")
     }
 
-
     if (!is.null(newSubjects)) {
       # add overall treatment
       newSubjects2 <- newSubjects %>%
-        dplyr::bind_rows(newSubjects %>% dplyr::mutate(treatment = 9999))
+        dplyr::bind_rows(newSubjects %>% dplyr::mutate(
+          treatment = 9999, treatment_description = "Overall"))
+
+      if (is.null(df)) {
+        sum_by_trt <- newSubjects2 %>%
+          dplyr::group_by(.data$treatment, .data$treatment_description) %>%
+          dplyr::slice(dplyr::n()) %>%
+          dplyr::mutate(n0 = 0, d0 = 0, c0 = 0, r0 = 0) %>%
+          dplyr::select(.data$treatment, .data$treatment_description,
+                        .data$n0, .data$d0, .data$c0, .data$r0)
+      }
 
       # predicted number of subjects enrolled by treatment after cutoff
       dfb1 <- dplyr::tibble(t = t) %>%
         dplyr::cross_join(newSubjects2) %>%
-        dplyr::group_by(.data$treatment, .data$t, .data$draw) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description,
+                        .data$t, .data$draw) %>%
         dplyr::summarise(nenrolled = sum(.data$arrivalTime <= .data$t),
                          .groups = "drop_last") %>%
-        dplyr::left_join(sum_by_trt, by = "treatment") %>%
+        dplyr::left_join(sum_by_trt,
+                         by = c("treatment", "treatment_description")) %>%
         dplyr::mutate(nenrolled = .data$nenrolled + .data$n0) %>%
         dplyr::summarise(n = quantile(.data$nenrolled, probs = 0.5),
                          lower = quantile(.data$nenrolled, probs = plower),
@@ -491,13 +538,13 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
 
     if (!is.null(df)) {
       # day 1
-      df0 <- dplyr::tibble(treatment = c(1:ngroups, 9999),
-                           t = 1, n = 0, lower = NA, upper = NA,
-                           mean = 0, var = 0)
+      df0 <- sum_by_trt %>%
+        dplyr::select(.data$treatment, .data$treatment_description) %>%
+        dplyr::mutate(t = 1, n = 0, lower = NA, upper = NA, mean = 0, var = 0)
 
       # arrival time for subjects already enrolled before data cut
       dfa1 <- df2 %>%
-        dplyr::group_by(.data$treatment) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description) %>%
         dplyr::arrange(.data$randdt) %>%
         dplyr::mutate(t = as.numeric(.data$randdt - trialsdt + 1),
                       n = dplyr::row_number()) %>%
@@ -506,11 +553,13 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
         dplyr::bind_rows(sum_by_trt %>%
                            dplyr::mutate(t = t0, n = n0, lower = NA,
                                          upper = NA, mean = n0, var = 0)) %>%
-        dplyr::select(.data$treatment, .data$t, .data$n, .data$lower,
-                      .data$upper, .data$mean, .data$var) %>%
-        dplyr::group_by(.data$treatment, .data$t) %>%
+        dplyr::select(.data$treatment, .data$treatment_description,
+                      .data$t, .data$n, .data$lower, .data$upper,
+                      .data$mean, .data$var) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description,
+                        .data$t) %>%
         dplyr::slice(dplyr::n()) %>%
-        dplyr::group_by(.data$treatment)
+        dplyr::group_by(.data$treatment, .data$treatment_description)
     }
 
 
@@ -538,7 +587,7 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
     }
 
     enroll_pred_df <- enroll_pred_df %>%
-      dplyr::arrange(.data$treatment, .data$t)
+      dplyr::arrange(.data$treatment, .data$treatment_description, .data$t)
   }
 
 
@@ -579,9 +628,10 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
   }
 
   newEvents = dplyr::as_tibble(matrix(
-    nrow = nreps*r0 + m1, ncol = 6,
-    dimnames = list(NULL, c("draw", "arrivalTime", "treatment", "time",
-                            "event", "dropout"))))
+    nrow = nreps*r0 + m1, ncol = 8,
+    dimnames = list(NULL, c("draw", "usubjid", "arrivalTime",
+                            "treatment", "treatment_description",
+                            "time", "event", "dropout"))))
 
   offset = 0
   for (i in 1:nreps) {
@@ -594,31 +644,42 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
 
     m = r0 + n1
 
-    # arrival time, treatment, and time offset for new subjects
+    # usubjid, arrival time, treatment, and time offset for new subjects
     if (n1 > 0) {
+      usubjidNew = newSubjects$usubjid[newSubjects$draw == i]
       arrivalTimeNew = newSubjects$arrivalTime[newSubjects$draw == i]
       treatmentNew = newSubjects$treatment[newSubjects$draw == i]
+      treatment_descriptionNew = newSubjects$treatment_description[
+        newSubjects$draw == i]
       time0New = rep(0, n1)
     }
 
     # concatenate ongoing and new subjects
     if (r0 == 0 && n1 > 0) {  # design stage
+      usubjid = usubjidNew
       arrivalTime = arrivalTimeNew
       treatment = treatmentNew
+      treatment_description = treatment_descriptionNew
       time0 = time0New
     } else if (r0 > 0 && n1 > 0) { # enrollment stage
+      usubjid = c(usubjidOngoing, usubjidNew)
       arrivalTime = c(arrivalTimeOngoing, arrivalTimeNew)
       treatment = c(treatmentOngoing, treatmentNew)
+      treatment_description = c(treatment_descriptionOngoing,
+                                treatment_descriptionNew)
       time0 = c(time0Ongoing, time0New)
     } else if (r0 > 0 && n1 == 0) { # follow-up stage
+      usubjid = usubjidOngoing
       arrivalTime = arrivalTimeOngoing
       treatment = treatmentOngoing
+      treatment_description = treatment_descriptionOngoing
       time0 = time0Ongoing
     }
 
 
     # draw event time for ongoing and new subjects
     survivalTime = rep(NA, m)
+
     for (j in 1:ngroups) {
       cols = which(treatment == j)
       ncols = length(cols)
@@ -630,10 +691,16 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
           rate = exp(theta2[[j]][i,])
           survivalTime[cols] = rexp(ncols, rate) + time0[cols]
         } else if (model == "weibull") {
-          shape = exp(theta2[[j]][i,1])
-          scale = exp(theta2[[j]][i,2])
+          shape = exp(-theta2[[j]][i,2])
+          scale = exp(theta2[[j]][i,1])
           survivalTime[cols] = (rexp(ncols)*scale^shape +
                                   time0[cols]^shape)^(1/shape)
+        } else if (model == "log-logistic") {
+          location = theta2[[j]][i,1]
+          scale = exp(theta2[[j]][i,2])
+          p = plogis((log(time0[cols]) - location)/scale, lower.tail = F)
+          survivalTime[cols] = exp(location + scale*qlogis(
+            runif(ncols)*p, lower.tail = F))
         } else if (model == "log-normal") {
           meanlog = theta2[[j]][i,1]
           sdlog = exp(theta2[[j]][i,2])
@@ -661,15 +728,15 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
           survivalTime[cols] = u[j1] + (rhs - psum[j1])/lambda[j1]
         } else if (model == "model averaging") {
           theta = theta2[[j]][i,]
-          shape = exp(theta[1])
-          scale = exp(theta[2])
+          shape = exp(-theta[2])
+          scale = exp(theta[1])
           meanlog = theta[3]
           sdlog = exp(theta[4])
           w1 = event_fit[[j]]$w1
 
           # draw component indicator
-          p1 = w1*pweibull(time0[cols], shape, scale, lower.tail = FALSE)
-          p2 = (1-w1)*plnorm(time0[cols], meanlog, sdlog, lower.tail = FALSE)
+          p1 = w1*pweibull(time0[cols], shape, scale, lower.tail = F)
+          p2 = (1-w1)*plnorm(time0[cols], meanlog, sdlog, lower.tail = F)
           w = (runif(ncols) < p1/(p1+p2))
           nw1 = sum(w)
           nw0 = ncols - nw1
@@ -691,12 +758,11 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
           scale = event_fit[[j]]$scale
 
           st0 = flexsurv::psurvspline(
-            time0[cols], gamma, knots = knots, scale = scale,
-            lower.tail = FALSE)
+            time0[cols], gamma, knots = knots, scale = scale, lower.tail = F)
 
           survivalTime[cols] = flexsurv::qsurvspline(
             runif(ncols)*st0, gamma, knots = knots, scale = scale,
-            lower.tail = FALSE)
+            lower.tail = F)
         }
       }
     }
@@ -719,10 +785,16 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
             rate = exp(theta3[[j]][i,])
             dropoutTime[cols] = rexp(ncols, rate) + time0[cols]
           } else if (model == "weibull") {
-            shape = exp(theta3[[j]][i,1])
-            scale = exp(theta3[[j]][i,2])
+            shape = exp(-theta3[[j]][i,2])
+            scale = exp(theta3[[j]][i,1])
             dropoutTime[cols] = (rexp(ncols)*scale^shape +
                                    time0[cols]^shape)^(1/shape)
+          } else if (model == "log-logistic") {
+            location = theta3[[j]][i,1]
+            scale = exp(theta3[[j]][i,2])
+            p = plogis((log(time0[cols]) - location)/scale, lower.tail = F)
+            dropoutTime[cols] = exp(location + scale*qlogis(
+              runif(ncols)*p, lower.tail = F))
           } else if (model == "log-normal") {
             meanlog = theta3[[j]][i,1]
             sdlog = exp(theta3[[j]][i,2])
@@ -748,6 +820,44 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
             # find the interval containing time
             j1 = findInterval(rhs, psum)
             dropoutTime[cols] = u[j1] + (rhs - psum[j1])/lambda[j1]
+          } else if (model == "model averaging") {
+            theta = theta3[[j]][i,]
+            shape = exp(-theta[2])
+            scale = exp(theta[1])
+            meanlog = theta[3]
+            sdlog = exp(theta[4])
+            w1 = dropout_fit[[j]]$w1
+
+            # draw component indicator
+            p1 = w1*pweibull(time0[cols], shape, scale, lower.tail = F)
+            p2 = (1-w1)*plnorm(time0[cols], meanlog, sdlog, lower.tail = F)
+            w = (runif(ncols) < p1/(p1+p2))
+            nw1 = sum(w)
+            nw0 = ncols - nw1
+
+            # draw from the corresponding component distribution
+            if (nw1 > 0) {
+              dropoutTime[cols][w==1] = (rexp(nw1)*scale^shape +
+                                           time0[cols][w==1]^shape)^(1/shape)
+            }
+
+            if (nw0 > 0) {
+              dropoutTime[cols][w==0] = exp(tmvtnsim::rtnorm(
+                mean = rep(meanlog, nw0), sd = sdlog,
+                lower = log(time0[cols][w==0]), upper = rep(Inf, nw0)))
+            }
+          } else if (model == "spline") {
+            gamma = theta3[[j]][i,]
+            knots = dropout_fit[[j]]$knots
+            scale = dropout_fit[[j]]$scale
+
+            st0 = flexsurv::psurvspline(
+              time0[cols], gamma, knots = knots, scale = scale,
+              lower.tail = F)
+
+            dropoutTime[cols] = flexsurv::qsurvspline(
+              runif(ncols)*st0, gamma, knots = knots, scale = scale,
+              lower.tail = F)
           }
         }
       }
@@ -783,8 +893,10 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
     # fill out the ith block of output data frame
     index = offset + (1:m)
     newEvents[index, "draw"] = i
+    newEvents[index, "usubjid"] = usubjid
     newEvents[index, "arrivalTime"] = arrivalTime
     newEvents[index, "treatment"] = treatment
+    newEvents[index, "treatment_description"] = treatment_description
     newEvents[index, "time"] = time
     newEvents[index, "event"] = event
     newEvents[index, "dropout"] = dropout
@@ -800,7 +912,8 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
     # combined stopped, ongoing and new subjects
     allSubjects <- dplyr::tibble(draw = 1:nreps) %>%
       dplyr::cross_join(stoppedSubjects) %>%
-      dplyr::select(.data$draw, .data$arrivalTime, .data$treatment,
+      dplyr::select(.data$draw, .data$usubjid, .data$arrivalTime,
+                    .data$treatment, .data$treatment_description,
                     .data$time, .data$event, .data$dropout,
                     .data$totalTime) %>%
       dplyr::bind_rows(newEvents)
@@ -809,7 +922,8 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
   }
 
   # remove the dummy treatment from newEvents
-  if (!by_treatment) newEvents <- newEvents %>% dplyr::select(-treatment)
+  if (!by_treatment) newEvents <- newEvents %>%
+    dplyr::select(-c(treatment, treatment_description))
 
   # A general quantile method if there are data sets not reaching target_d
   # Find t such that sum(I{D_i(t) < target_d}, {i, 1, nreps}) / nreps = q.
@@ -823,20 +937,31 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
     mean(sumdata$n < target_d)
   }
 
-  # obtain the quantiles
-  q = 1 - c(0.5, plower, pupper)
-  pred_day = rep(NA, length(q))
   tmax = max(newEvents$totalTime[newEvents$event==1])
-  for (j in 1:length(q)) {
-    # check if the quantile can be estimated from observed data
-    if (sdf(tmax, target_d, d0, newEvents) <= q[j]) {
-      pred_day[j] = uniroot(function(x)
-        sdf(x, target_d, d0, newEvents) - q[j],
-        c(tp, tmax), tol = 1)$root
-      pred_day[j] = ceiling(pred_day[j])
+
+  # obtain the quantiles
+  if (sdf(tmax, target_d, d0, newEvents) == 0) {
+    new1 <- newEvents %>%
+      dplyr::group_by(.data$draw) %>%
+      dplyr::filter(.data$event == 1) %>%
+      dplyr::arrange(.data$draw, .data$totalTime) %>%
+      dplyr::filter(dplyr::row_number() == target_d - d0)
+    pred_day <- ceiling(quantile(new1$totalTime, c(0.5, plower, pupper)))
+  } else {
+    q = 1 - c(0.5, plower, pupper)
+    pred_day = rep(NA, length(q))
+    for (j in 1:length(q)) {
+      # check if the quantile can be estimated from observed data
+      if (sdf(tmax, target_d, d0, newEvents) <= q[j]) {
+        pred_day[j] = uniroot(function(x)
+          sdf(x, target_d, d0, newEvents) - q[j],
+          c(tp, tmax), tol = 1)$root
+        pred_day[j] = ceiling(pred_day[j])
+      }
     }
+    names(pred_day) <- names(quantile(1:100, c(0.5, plower, pupper)))
   }
-  names(pred_day) <- names(quantile(1:100, c(0.5, plower, pupper)))
+
 
   if (!is.null(df)) {
     pred_date <- as.Date(pred_day - 1, origin = trialsdt)
@@ -1036,7 +1161,8 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
           fill = "tonexty", fillcolor = ~parameter,
           line = list(width=0)) %>%
         plotly::add_lines(
-          data = dfb, x = ~date, y = ~n, color = ~parameter) %>%
+          data = dfb, x = ~date, y = ~n, color = ~parameter,
+          line = list(width=2)) %>%
         plotly::add_lines(
           data = dfa, x = ~date, y = ~n, color = ~parameter,
           line = list(shape="hv", width=2)) %>%
@@ -1085,7 +1211,8 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
           fill = "tonexty", fillcolor = ~parameter,
           line = list(width=0)) %>%
         plotly::add_lines(
-          data = dfs, x = ~t, y = ~n, color = ~parameter) %>%
+          data = dfs, x = ~t, y = ~n, color = ~parameter,
+          line = list(width=2)) %>%
         plotly::layout(
           xaxis = list(title = "Days since trial start", zeroline = FALSE),
           yaxis = list(zeroline = FALSE),
@@ -1108,12 +1235,14 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
   } else {  # by treatment
     # add overall treatment
     allSubjects2 <- allSubjects %>%
-      dplyr::bind_rows(allSubjects %>% dplyr::mutate(treatment = 9999))
+      dplyr::bind_rows(allSubjects %>% dplyr::mutate(
+        treatment = 9999, treatment_description = "Overall"))
 
     # number of events, dropouts, and ongoing subjects after data cut
     df1 = dplyr::tibble(t = t) %>%
       dplyr::cross_join(allSubjects2) %>%
-      dplyr::group_by(.data$treatment, .data$t, .data$draw) %>%
+      dplyr::group_by(.data$treatment, .data$treatment_description,
+                      .data$t, .data$draw) %>%
       dplyr::summarise(nevents = sum(.data$totalTime <= .data$t &
                                        .data$event == 1),
                        ndropouts = sum(.data$totalTime <= .data$t &
@@ -1152,85 +1281,93 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
 
     if (!is.null(df)) {
       # day 1
-      df0 <- dplyr::tibble(treatment = c(1:ngroups, 9999),
-                           t = 1, n = 0, lower = NA, upper = NA,
-                           mean = 0, var = 0)
+      df0 <- sum_by_trt %>%
+        dplyr::select(.data$treatment, .data$treatment_description) %>%
+        dplyr::mutate(t = 1, n = 0, lower = NA, upper = NA, mean = 0, var = 0)
+
 
       # observed number of events before data cut
       dfa2 <- df2 %>%
-        dplyr::group_by(.data$treatment) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description) %>%
         dplyr::arrange(.data$totalTime) %>%
         dplyr::mutate(t = .data$totalTime, n = cumsum(.data$event)) %>%
         dplyr::mutate(lower = NA, upper = NA, mean = .data$n, var = 0) %>%
-        dplyr::select(.data$treatment, .data$t, .data$n, .data$lower,
-                      .data$upper, .data$mean, .data$var) %>%
+        dplyr::select(.data$treatment, .data$treatment_description,
+                      .data$t, .data$n, .data$lower, .data$upper,
+                      .data$mean, .data$var) %>%
         dplyr::bind_rows(df0) %>%
         dplyr::filter(.data$t <= tp) %>%
-        dplyr::arrange(.data$treatment, .data$t)
+        dplyr::arrange(.data$treatment, .data$treatment_description, .data$t)
 
       # observed number of dropouts before data cut
       dfa3 <- df2 %>%
-        dplyr::group_by(.data$treatment) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description) %>%
         dplyr::arrange(.data$totalTime) %>%
         dplyr::mutate(t = .data$totalTime, n = cumsum(.data$dropout)) %>%
         dplyr::mutate(lower = NA, upper = NA, mean = .data$n, var = 0) %>%
-        dplyr::select(.data$treatment, .data$t, .data$n, .data$lower,
-                      .data$upper, .data$mean, .data$var) %>%
+        dplyr::select(.data$treatment, .data$treatment_description,
+                      .data$t, .data$n, .data$lower, .data$upper,
+                      .data$mean, .data$var) %>%
         dplyr::bind_rows(df0) %>%
         dplyr::filter(.data$t <= tp) %>%
-        dplyr::arrange(.data$treatment, .data$t)
+        dplyr::arrange(.data$treatment, .data$treatment_description, .data$t)
 
 
       # observed number of ongoing subjects before data cutoff
       dfa4 <- dplyr::tibble(t = t2) %>%
         dplyr::cross_join(df2) %>%
-        dplyr::group_by(.data$treatment, .data$t) %>%
-        dplyr::summarise(n = sum(.data$arrivalTime <= .data$t &
-                                   (.data$totalTime > .data$t |
-                                      (.data$event == 0 &
-                                         .data$dropout == 0))),
-                         .groups = "drop_last") %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description,
+                        .data$t) %>%
+        dplyr::summarise(
+          n = sum(.data$arrivalTime <= .data$t &
+                    (.data$totalTime > .data$t |
+                       (.data$event == 0 & .data$dropout == 0))),
+          .groups = "drop_last") %>%
         dplyr::mutate(lower = NA, upper = NA, mean = .data$n, var = 0) %>%
-        dplyr::select(.data$treatment, .data$t, .data$n, .data$lower,
-                      .data$upper, .data$mean, .data$var) %>%
+        dplyr::select(.data$treatment, .data$treatment_description,
+                      .data$t, .data$n, .data$lower, .data$upper,
+                      .data$mean, .data$var) %>%
         dplyr::bind_rows(df0) %>%
         dplyr::filter(.data$t <= tp) %>%
-        dplyr::arrange(.data$treatment, .data$t)
+        dplyr::arrange(.data$treatment, .data$treatment_description, .data$t)
 
 
       # add time tp
       dfa2tp <- dfa2 %>%
-        dplyr::group_by(.data$treatment) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description) %>%
         dplyr::slice(dplyr::n()) %>%
         dplyr::mutate(t = tp)
 
       dfa2 <- dfa2 %>%
         dplyr::bind_rows(dfa2tp) %>%
-        dplyr::group_by(.data$treatment, .data$t) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description,
+                        .data$t) %>%
         dplyr::slice(dplyr::n()) %>%
         dplyr::ungroup()
 
 
       dfa3tp <- dfa3 %>%
-        dplyr::group_by(.data$treatment) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description) %>%
         dplyr::slice(dplyr::n()) %>%
         dplyr::mutate(t = tp)
 
       dfa3 <- dfa3 %>%
         dplyr::bind_rows(dfa3tp) %>%
-        dplyr::group_by(.data$treatment, .data$t) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description,
+                        .data$t) %>%
         dplyr::slice(dplyr::n()) %>%
         dplyr::ungroup()
 
 
       dfa4tp <- dfa4 %>%
-        dplyr::group_by(.data$treatment) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description) %>%
         dplyr::slice(dplyr::n()) %>%
         dplyr::mutate(t = tp)
 
       dfa4 <- dfa4 %>%
         dplyr::bind_rows(dfa4tp) %>%
-        dplyr::group_by(.data$treatment, .data$t) %>%
+        dplyr::group_by(.data$treatment, .data$treatment_description,
+                        .data$t) %>%
         dplyr::slice(dplyr::n()) %>%
         dplyr::ungroup()
 
@@ -1264,13 +1401,13 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
     }
 
     event_pred_df <- event_pred_df %>%
-      dplyr::arrange(.data$treatment, .data$t)
+      dplyr::arrange(.data$treatment, .data$treatment_description, .data$t)
 
     dropout_pred_df <- dropout_pred_df %>%
-      dplyr::arrange(.data$treatment, .data$t)
+      dplyr::arrange(.data$treatment, .data$treatment_description, .data$t)
 
     ongoing_pred_df <- ongoing_pred_df %>%
-      dplyr::arrange(.data$treatment, .data$t)
+      dplyr::arrange(.data$treatment, .data$treatment_description, .data$t)
 
     dfs <- dplyr::tibble()
     if (showEnrollment) dfs <- dfs %>% dplyr::bind_rows(enroll_pred_df)
@@ -1294,7 +1431,7 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
         dfai <- dfa %>%
           dplyr::filter(.data$treatment == i)
 
-        g0 <- plotly::plot_ly() %>%
+        g[[(i+1) %% 9999]] <- plotly::plot_ly() %>%
           plotly::add_ribbons(
             data = dfbi, x = ~date, ymin = ~lower, ymax = ~upper,
             fill = "tonexty", fillcolor = ~parameter,
@@ -1312,10 +1449,17 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
             xaxis = list(title = "", zeroline = FALSE),
             yaxis = list(zeroline = FALSE),
             legend = list(x = 0, y = 1.05, yanchor = "bottom",
-                          orientation = 'h'))
+                          orientation = 'h')) %>%
+          plotly::layout(
+            annotations = list(
+              x = 0.5, y = 1,
+              text = paste0("<b>", dfsi$treatment_description[1], "</b>"),
+              xanchor = "center", yanchor = "bottom",
+              showarrow = FALSE, xref='paper', yref='paper'))
+
 
         if (tp < t0) {
-          g0 <- g0 %>%
+          g[[(i+1) %% 9999]] <- g[[(i+1) %% 9999]] %>%
             plotly::add_lines(
               x = rep(cutofftpdt, 2), y = range(dfsi$n),
               name = "prediction start",
@@ -1324,17 +1468,12 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
 
 
         if (i == 9999) {
-          g[[1]] <- g0 %>%
+          g[[1]] <- g[[1]] %>%
             plotly::layout(
               annotations = list(
                 x = cutoffdt, y = 0, text = 'cutoff', xanchor = "left",
                 yanchor = "bottom", font = list(size=12),
-                showarrow = FALSE)) %>%
-            plotly::layout(
-              annotations = list(
-                x = 0.5, y = 1, text = "<b>Overall</b>",
-                xanchor = "center", yanchor = "bottom",
-                showarrow = FALSE, xref='paper', yref='paper'))
+                showarrow = FALSE))
 
           if (tp < t0) {
             g[[1]] <- g[[1]] %>%
@@ -1358,13 +1497,6 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
                   yanchor = "bottom", font = list(size=12),
                   showarrow = FALSE))
           }
-        } else {
-          g[[i+1]] <- g0 %>%
-            plotly::layout(
-              annotations = list(
-                x = 0.5, y = 1, text = paste0("<b>treatment=", i, "</b>"),
-                xanchor = "center", yanchor = "bottom",
-                showarrow = FALSE, xref='paper', yref='paper'))
         }
       }
     } else { # prediction at design stage
@@ -1373,7 +1505,7 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
         dfsi <- dfs %>%
           dplyr::filter(.data$treatment == i)
 
-        g0 <- plotly::plot_ly() %>%
+        g[[(i+1) %% 9999]] <- plotly::plot_ly() %>%
           plotly::add_ribbons(
             data = dfsi, x = ~t, ymin = ~lower, ymax = ~upper,
             fill = "tonexty", fillcolor = ~parameter,
@@ -1385,16 +1517,16 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
             xaxis = list(title = "Days since trial start", zeroline = FALSE),
             yaxis = list(zeroline = FALSE),
             legend = list(x = 0, y = 1.05, yanchor = "bottom",
-                          orientation = 'h'))
+                          orientation = 'h')) %>%
+          plotly::layout(
+            annotations = list(
+              x = 0.5, y = 1,
+              text = paste0("<b>", dfsi$treatment_description[1], "</b>"),
+              xanchor = "center", yanchor = "bottom",
+              showarrow = FALSE, xref='paper', yref='paper'))
+
 
         if (i == 9999) {
-          g[[1]] <- g0 %>%
-            plotly::layout(
-              annotations = list(
-                x = 0.5, y = 1, text = "<b>Overall</b>",
-                xanchor = "center", yanchor = "bottom",
-                showarrow = FALSE, xref='paper', yref='paper'))
-
           if (showEvent) {
             g[[1]]  <- g[[1]] %>%
               plotly::add_lines(
@@ -1408,13 +1540,6 @@ predictEvent <- function(df = NULL, target_d, newSubjects = NULL,
                   yanchor = "bottom", font = list(size=12),
                   showarrow = FALSE))
           }
-        } else {
-          g[[i+1]] <- g0 %>%
-            plotly::layout(
-              annotations = list(
-                x = 0.5, y = 1, text = paste0("<b>treatment=", i, "</b>"),
-                xanchor = "center", yanchor = "bottom",
-                showarrow = FALSE, xref='paper', yref='paper'))
         }
       }
     }
